@@ -19,6 +19,139 @@
  * Boston, MA 02111-1307, USA.
  */
 
+/**
+ * SECTION:pangocairo
+ * @short_description:Rendering with the Cairo backend
+ * @title:Cairo Rendering
+ *
+ * The <ulink url="http://cairographics.org">Cairo library</ulink> is a
+ * vector graphics library with a powerful rendering model. It has such
+ * features as anti-aliased primitives, alpha-compositing, and
+ * gradients. Multiple backends for Cairo are available, to allow
+ * rendering to images, to PDF files, and to the screen on X and on other
+ * windowing systems. The functions in this section allow using Pango
+ * to render to Cairo surfaces.
+ *
+ * Using Pango with Cairo is straightforward. A #PangoContext created
+ * with pango_cairo_font_map_create_context() can be used on any
+ * Cairo context (cairo_t), but needs to be updated to match the
+ * current transformation matrix and target surface of the Cairo context
+ * using pango_cairo_update_context(). The convenience functions
+ * pango_cairo_create_layout() and pango_cairo_update_layout() handle
+ * the common case where the program doesn't need to manipulate the
+ * properties of the #PangoContext.
+ *
+ * When you get the metrics of a layout or of a piece of a layout using
+ * functions such as pango_layout_get_extents(), the reported metrics
+ * are in user-space coordinates. If a piece of text is 10 units long,
+ * and you call cairo_scale (cr, 2.0), it still is more-or-less 10
+ * units long. However, the results will be affected by hinting
+ * (that is, the process of adjusting the text to look good on the
+ * pixel grid), so you shouldn't assume they are completely independent
+ * of the current transformation matrix. Note that the basic metrics
+ * functions in Pango report results in integer Pango units. To get
+ * to the floating point units used in Cairo divide by %PANGO_SCALE.
+ *
+ * <example id="rotated-example">
+ * <title>Using Pango with Cairo</title>
+ *  <programlisting>
+ * #include <math.h>
+ * #include <pango/pangocairo.h>
+ *
+ * static void
+ * draw_text (cairo_t *cr)
+ * {
+ * #define RADIUS 150
+ * #define N_WORDS 10
+ * #define FONT "Sans Bold 27"
+ *
+ *   PangoLayout *layout;
+ *   PangoFontDescription *desc;
+ *   int i;
+ *
+ *   /&ast; Center coordinates on the middle of the region we are drawing
+ *    &ast;/
+ *   cairo_translate (cr, RADIUS, RADIUS);
+ *
+ *   /&ast; Create a PangoLayout, set the font and text &ast;/
+ *   layout = pango_cairo_create_layout (cr);
+ *
+ *   pango_layout_set_text (layout, "Text", -1);
+ *   desc = pango_font_description_from_string (FONT);
+ *   pango_layout_set_font_description (layout, desc);
+ *   pango_font_description_free (desc);
+ *
+ *   /&ast; Draw the layout N_WORDS times in a circle &ast;/
+ *   for (i = 0; i &lt; N_WORDS; i++)
+ *     {
+ *       int width, height;
+ *       double angle = (360. * i) / N_WORDS;
+ *       double red;
+ *
+ *       cairo_save (cr);
+ *
+ *       /&ast; Gradient from red at angle == 60 to blue at angle == 240 &ast;/
+ *       red   = (1 + cos ((angle - 60) * G_PI / 180.)) / 2;
+ *       cairo_set_source_rgb (cr, red, 0, 1.0 - red);
+ *
+ *       cairo_rotate (cr, angle * G_PI / 180.);
+ *
+ *       /&ast; Inform Pango to re-layout the text with the new transformation &ast;/
+ *       pango_cairo_update_layout (cr, layout);
+ *
+ *       pango_layout_get_size (layout, &amp;width, &amp;height);
+ *       cairo_move_to (cr, - ((double)width / PANGO_SCALE) / 2, - RADIUS);
+ *       pango_cairo_show_layout (cr, layout);
+ *
+ *       cairo_restore (cr);
+ *     }
+ *
+ *   /&ast; free the layout object &ast;/
+ *   g_object_unref (layout);
+ * }
+ *
+ * int main (int argc, char **argv)
+ * {
+ *   cairo_t *cr;
+ *   char *filename;
+ *   cairo_status_t status;
+ *   cairo_surface_t *surface;
+ *
+ *   if (argc != 2)
+ *     {
+ *       g_printerr ("Usage: cairosimple OUTPUT_FILENAME\n");
+ *       return 1;
+ *     }
+ *
+ *      filename = argv[1];
+ *
+ *   surface = cairo_image_surface_create (CAIRO_FORMAT_ARGB32,
+ *                                         2 * RADIUS, 2 * RADIUS);
+ *   cr = cairo_create (surface);
+ *
+ *   cairo_set_source_rgb (cr, 1.0, 1.0, 1.0);
+ *   cairo_paint (cr);
+ *   draw_text (cr);
+ *   cairo_destroy (cr);
+ *
+ *   status = cairo_surface_write_to_png (surface, filename);
+ *   cairo_surface_destroy (surface);
+ *
+ *   if (status != CAIRO_STATUS_SUCCESS)
+ *     {
+ *       g_printerr ("Could not save png to '%s'\n", filename);
+ *       return 1;
+ *     }
+ *
+ *   return 0;
+ * }
+ * </programlisting>
+ * </example>
+ * <figure id="rotated-example-output">
+ *   <title>Output of <xref linkend="rotated-example"/></title>
+ *   <graphic fileref="rotated-text.png" format="PNG"/>
+ * </figure>
+ */
 #include "config.h"
 
 #include <math.h>
@@ -58,12 +191,38 @@ set_color (PangoCairoRenderer *crenderer,
 	   PangoRenderPart     part)
 {
   PangoColor *color = pango_renderer_get_color ((PangoRenderer *) (crenderer), part);
+  guint16 a = pango_renderer_get_alpha ((PangoRenderer *) (crenderer), part);
+  gdouble red, green, blue, alpha;
+
+  if (!a && !color)
+    return;
 
   if (color)
-    cairo_set_source_rgb (crenderer->cr,
-			  color->red / 65535.,
-			  color->green / 65535.,
-			  color->blue / 65535.);
+    {
+      red = color->red / 65535.;
+      green = color->green / 65535.;
+      blue = color->blue / 65535.;
+      alpha = 1.;
+    }
+  else
+    {
+      cairo_pattern_t *pattern = cairo_get_source (crenderer->cr);
+
+      if (pattern && cairo_pattern_get_type (pattern) == CAIRO_PATTERN_TYPE_SOLID)
+        cairo_pattern_get_rgba (pattern, &red, &green, &blue, &alpha);
+      else
+        {
+          red = 0.;
+          green = 0.;
+          blue = 0.;
+          alpha = 1.;
+        }
+    }
+
+  if (a)
+    alpha = a / 65535.;
+
+  cairo_set_source_rgba (crenderer->cr, red, green, blue, alpha);
 }
 
 /* note: modifies crenderer->cr without doing cairo_save/restore() */
@@ -128,7 +287,7 @@ _pango_cairo_renderer_draw_frame (PangoCairoRenderer *crenderer,
 	  alpha = G_PI_2 - alpha;
 	  tan_alpha2 = tan (alpha * .5);
 	  if (tan_alpha2 < 1e-5 || (sy = d2 / tan_alpha2, 2. * sy > height - d))
-	    sy = (width - d) * .5;
+	    sy = (height - d) * .5;
 
 	  cos_alpha = cos (alpha);
 	  if (cos_alpha < 1e-5 || (sx = d2 / cos_alpha, 2. * sx > width - d))
@@ -208,6 +367,7 @@ _pango_cairo_renderer_draw_unknown_glyph (PangoCairoRenderer *crenderer,
   double x0, y0;
   int row, col;
   int rows, cols;
+  double width, lsb;
   char hexbox_string[2] = {0, 0};
   PangoCairoFontHexBoxInfo *hbi;
   gunichar ch;
@@ -236,10 +396,14 @@ _pango_cairo_renderer_draw_unknown_glyph (PangoCairoRenderer *crenderer,
       g_snprintf (buf, sizeof(buf), (ch > 0xffff) ? "%06X" : "%04X", ch);
     }
 
+  width = (3 * hbi->pad_x + cols * (hbi->digit_width + hbi->pad_x));
+  lsb = ((double)gi->geometry.width / PANGO_SCALE - width) * .5;
+  lsb = floor (lsb / hbi->pad_x) * hbi->pad_x;
+
   _pango_cairo_renderer_draw_frame (crenderer,
-				    cx + hbi->pad_x * 1.5,
+				    cx + lsb + .5 * hbi->pad_x,
 				    cy + hbi->box_descent - hbi->box_height + hbi->pad_y * 0.5,
-				    (double)gi->geometry.width / PANGO_SCALE - 3 * hbi->pad_x,
+				    width - hbi->pad_x,
 				    (hbi->box_height - hbi->pad_y),
 				    hbi->line_width,
 				    invalid_input);
@@ -247,7 +411,7 @@ _pango_cairo_renderer_draw_unknown_glyph (PangoCairoRenderer *crenderer,
   if (invalid_input)
     goto done;
 
-  x0 = cx + hbi->pad_x * 3.0;
+  x0 = cx + lsb + hbi->pad_x * 2;
   y0 = cy + hbi->box_descent - hbi->pad_y * 2;
 
   for (row = 0; row < rows; row++)
@@ -698,7 +862,7 @@ pango_cairo_renderer_class_init (PangoCairoRendererClass *klass)
   renderer_class->draw_shape = pango_cairo_renderer_draw_shape;
 }
 
-static PangoCairoRenderer *cached_renderer = NULL;
+static PangoCairoRenderer *cached_renderer = NULL; /* MT-safe */
 G_LOCK_DEFINE_STATIC (cached_renderer);
 
 static PangoCairoRenderer *
@@ -1035,7 +1199,7 @@ pango_cairo_show_error_underline (cairo_t *cr,
 }
 
 /**
- * pango_cairo_glyph_string_path
+ * pango_cairo_glyph_string_path:
  * @cr: a Cairo context
  * @font: a #PangoFont from a #PangoCairoFontMap
  * @glyphs: a #PangoGlyphString
